@@ -13,17 +13,17 @@
  * more details.
  *
  * You should have received a copy of the GNU General Public License along with
- * this program; if not, write to the Free Software Foundation, Inc., 51
- * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ * this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include <ctype.h>
 #include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
 
 #include "ll.h"
 #include "decodekey.h"
-#include "log.h"
+#include "keystructs.h"
 #include "wordlist.h"
 
 /**
@@ -106,6 +106,7 @@ struct ll *makewordlistfromkey(struct ll *wordlist,
 				wordlist = lladd(wordlist, strdup(wl->object));
 			}
 		}
+		llfree(words, NULL);
 		free(uids[i]);
 		uids[i] = NULL;
 	}
